@@ -26,6 +26,8 @@
 #include <icd/osso-ic-dbus.h>
 #include "connectnow-home-widget.h"
 #include <math.h>
+#include <libintl.h>
+#include <locale.h>
 	
 HD_DEFINE_PLUGIN_MODULE(ConnectNowHomePlugin, connect_now_home_plugin, HD_TYPE_HOME_PLUGIN_ITEM)
 
@@ -55,9 +57,10 @@ connect_now_save_settings(ConnectNowHomePlugin* desktop_plugin)
   keyFile = g_key_file_new();
   filename = g_strconcat (g_get_home_dir(), CONNECT_NOW_HOME_PLUGIN_SETTINGS_FILE, NULL);
   g_key_file_load_from_file (keyFile, filename, G_KEY_FILE_KEEP_COMMENTS, NULL);
-
-  g_key_file_set_string(keyFile, desktop_plugin->iD, "connection_id", desktop_plugin->connection_id);
-  g_key_file_set_string(keyFile, desktop_plugin->iD, "connection_name", desktop_plugin->connection_name);
+  if(desktop_plugin->connection_id)
+    g_key_file_set_string(keyFile, desktop_plugin->iD, "connection_id", desktop_plugin->connection_id);
+  if(desktop_plugin->connection_name)
+    g_key_file_set_string(keyFile, desktop_plugin->iD, "connection_name", desktop_plugin->connection_name);
   g_key_file_set_integer(keyFile, desktop_plugin->iD, "widget_size", desktop_plugin->widget_size);
 
   fileData = g_key_file_to_data (keyFile, &size, NULL);
@@ -82,18 +85,23 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
 
   GtkWidget *dialog;
   GtkListStore *connection_list;
-  GtkCellRenderer *renderer = NULL;
 
+  GtkCellRenderer *renderer = NULL;
   GtkWidget* selector;
   GSList* iap_list;
   GSList* iap_entry;
+
   GtkWidget* sizeScale = NULL;
+
   HildonTouchSelectorColumn* column = NULL;
+  GtkWidget* connectionButton = hildon_picker_button_new(HILDON_SIZE_AUTO, 
+							 HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+  GtkWidget* displayNameEntry = hildon_entry_new(HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH);
 
   dialog = gtk_dialog_new_with_buttons("Settings",
 				       NULL,
 				       0,
-				       "Save",
+				       dgettext("hildon-libs", "wdgt_bd_done"),
 				       GTK_RESPONSE_ACCEPT,
 				       NULL);
   
@@ -102,6 +110,16 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
   sizeScale = gtk_hscale_new_with_range(50, 200, 10);
   gtk_scale_set_digits(GTK_SCALE(sizeScale), 0);
   gtk_range_set_value(GTK_RANGE(sizeScale), desktop_plugin->widget_size);
+  renderer = gtk_cell_renderer_text_new();
+
+  column = hildon_touch_selector_append_column(HILDON_TOUCH_SELECTOR(selector),
+					       GTK_TREE_MODEL(connection_list),
+					       renderer, "text", 0, NULL);
+
+  hildon_touch_selector_column_set_text_column(column, 0);
+  hildon_touch_selector_set_column_selection_mode (HILDON_TOUCH_SELECTOR (selector),
+						   HILDON_TOUCH_SELECTOR_SELECTION_MODE_SINGLE);
+
 
   iap_list = con_ic_connection_get_all_iaps(desktop_plugin->con);
 
@@ -113,29 +131,23 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
     GtkTreeIter iter;
     gtk_list_store_append (connection_list, &iter);
     gtk_list_store_set(connection_list, &iter, 0, name, 1, iap, -1);
+    if(g_strcmp0(iap, desktop_plugin->connection_id)==0)
+    {
+      hildon_touch_selector_select_iter(HILDON_TOUCH_SELECTOR(selector),
+					0,
+					&iter,
+					FALSE);
+      hildon_button_set_value(HILDON_BUTTON(connectionButton), name);
+    }
     g_object_unref(item);
   }
   g_slist_free(iap_list);
-
-  renderer = gtk_cell_renderer_text_new();
-
-  column = hildon_touch_selector_append_column(HILDON_TOUCH_SELECTOR(selector),
-					       GTK_TREE_MODEL(connection_list),
-					       renderer, "text", 0, NULL);
-
-  hildon_touch_selector_column_set_text_column(column, 0);
-  hildon_touch_selector_set_column_selection_mode (HILDON_TOUCH_SELECTOR (selector),
-						   HILDON_TOUCH_SELECTOR_SELECTION_MODE_SINGLE);
-
-  GtkWidget* connectionButton = hildon_picker_button_new(HILDON_SIZE_THUMB_HEIGHT, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+  
   hildon_picker_button_set_selector(HILDON_PICKER_BUTTON(connectionButton),
 				    HILDON_TOUCH_SELECTOR(selector));
 
-  GtkWidget* content = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
   GtkWidget* displayNameBox = gtk_hbox_new(TRUE, 3);
   GtkWidget* displayNameLabel = gtk_label_new("Displayname:");
-  GtkWidget* displayNameEntry = hildon_entry_new(HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH);
-  gtk_entry_set_text(GTK_ENTRY(displayNameEntry), desktop_plugin->connection_name);
 
   g_signal_connect(G_OBJECT(connectionButton),"value-changed",
 		   G_CALLBACK(connect_now_settings_connection_changed),
@@ -150,6 +162,7 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
   gtk_box_pack_start(GTK_BOX(connectionNameBox), connectionNameLabel, TRUE, TRUE, 3);
   gtk_box_pack_end(GTK_BOX(connectionNameBox), connectionButton, TRUE, TRUE, 3);
 
+  GtkWidget* content = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
   gtk_box_pack_start(GTK_BOX(content), displayNameBox, FALSE, FALSE, 3);
   gtk_box_pack_start(GTK_BOX(content), connectionNameBox, FALSE, FALSE, 3);
   gtk_box_pack_end(GTK_BOX(content), sizeScale, FALSE, FALSE, 3);
@@ -157,43 +170,50 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
 
   if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
   {
-    GtkTreeIter iter;
-    gchar *id;
-    if (hildon_touch_selector_get_selected(HILDON_TOUCH_SELECTOR(selector), 0, &iter))
-    {
-      const gchar* displayName = hildon_entry_get_text(HILDON_ENTRY(displayNameEntry));
-      gtk_tree_model_get(GTK_TREE_MODEL(connection_list), &iter, 1,  &id,  -1);
+    const gchar* displayName = hildon_entry_get_text(HILDON_ENTRY(displayNameEntry));
+    const gchar* connection_name = hildon_button_get_value(HILDON_BUTTON(connectionButton));
 
-      g_free(desktop_plugin->connection_name);
-      g_free(desktop_plugin->connection_id);
-      desktop_plugin->connection_id = id;
-      
-      if(displayName!=NULL && displayName[0]!='\0')
-      {
-	desktop_plugin->connection_name = g_strdup(displayName);
-      }
-      else
-      {
-	gtk_tree_model_get(GTK_TREE_MODEL(connection_list), &iter, 0, &(desktop_plugin->connection_name),  -1);
-      }
-      desktop_plugin->widget_size = gtk_range_get_value(GTK_RANGE(sizeScale));
-      gtk_window_resize(GTK_WINDOW(desktop_plugin),
-			(int)(desktop_plugin->widget_size),
-			(int)(desktop_plugin->widget_size));
-      if(g_strcmp0(desktop_plugin->current_connection_id,
-		   desktop_plugin->connection_id)==0)
-      {
-	desktop_plugin->this_connection_state = CON_IC_STATUS_CONNECTED;
-      }
-      else
-      {
-	desktop_plugin->this_connection_state = CON_IC_STATUS_DISCONNECTED;
-      }
-      connect_now_save_settings(desktop_plugin);
-      gtk_widget_queue_draw(GTK_WIDGET(desktop_plugin));
+    g_free(desktop_plugin->connection_name);
+    
+    if(displayName!=NULL && displayName[0]!='\0')
+    {
+      desktop_plugin->connection_name = g_strdup(displayName);
     }
+    else
+    {
+      desktop_plugin->connection_name = g_strdup(connection_name);
+    }
+    GtkTreeIter iter;
+    gboolean selected = hildon_touch_selector_get_selected(HILDON_TOUCH_SELECTOR(selector),
+							   0, &iter);
+    if(selected)
+    {
+      gchar* iap;
+      gtk_tree_model_get(GTK_TREE_MODEL(connection_list),
+			 &iter,
+			 1, &iap,
+			 -1);
+      g_free(desktop_plugin->connection_id);
+      desktop_plugin->connection_id = iap;
+    }
+    desktop_plugin->widget_size = gtk_range_get_value(GTK_RANGE(sizeScale));
+    gtk_window_resize(GTK_WINDOW(desktop_plugin),
+		      (int)(desktop_plugin->widget_size),
+		      (int)(desktop_plugin->widget_size));
+    if(g_strcmp0(desktop_plugin->current_connection_id,
+		 desktop_plugin->connection_id)==0)
+    {
+      desktop_plugin->this_connection_state = CON_IC_STATUS_CONNECTED;
+    }
+    else
+    {
+      desktop_plugin->this_connection_state = CON_IC_STATUS_DISCONNECTED;
+    }
+    connect_now_save_settings(desktop_plugin);
+    gtk_widget_queue_draw(GTK_WIDGET(desktop_plugin));
   }
   gtk_widget_destroy (dialog);
+  g_object_unref(connection_list);
 }
 
 void 
@@ -515,7 +535,8 @@ connect_now_expose(GtkWidget* widget, GdkEventExpose *event)
     cairo_stroke(cr);
 
     pangoLayout = pango_cairo_create_layout(cr);
-    pango_layout_set_text(pangoLayout, CONNECT_NOW_HOME_PLUGIN(widget)->connection_name, -1);
+    if(CONNECT_NOW_HOME_PLUGIN(widget)->connection_name)
+      pango_layout_set_text(pangoLayout, CONNECT_NOW_HOME_PLUGIN(widget)->connection_name, -1);
     const char* font = "Sans 12";
     fontDescription = pango_font_description_from_string(font);
     pango_layout_set_font_description(pangoLayout, fontDescription);
@@ -569,8 +590,10 @@ connect_now_plugin_finalize(GObject *object)
   g_free(desktop_plugin->connectedImageName);
   g_free(desktop_plugin->disconnectedImageName);
   g_free(desktop_plugin->iD);
-  g_object_unref(desktop_plugin->connectedImage);
-  g_object_unref(desktop_plugin->disconnectedImage);
+  if(desktop_plugin->connectedImage)
+    g_object_unref(desktop_plugin->connectedImage);
+  if(desktop_plugin->disconnectedImage)
+    g_object_unref(desktop_plugin->disconnectedImage);
   G_OBJECT_CLASS (connect_now_home_plugin_parent_class)->finalize (object);
 }
 
